<?php

namespace Drupal\data_export\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Database;
use Drupal\data_export\Service\DataExportService;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ExportTableNameForm extends FormBase
{
     /**
   * The Data Export Service.
   *
   * @var \Drupal\data_export\Service\DataExportService
   */
  protected $dataExportService;

  /**
   * Constructs a new ExportUsingCodeForm object.
   *
   * @param \Drupal\data_export\Service\DataExportService $data_export_service
   *   The Data Export Service.
   */
  public function __construct(DataExportService $data_export_service) {
    $this->dataExportService = $data_export_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('data_export.data_export_service')
    );
  }

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'export_table_name_form';
    }

    /**
     * Builds the form.
     */
   
     public function buildForm(array $form, FormStateInterface $form_state)
     {
         $form['table_name'] = [
             '#type' => 'textfield',
             '#title' => $this->t('Table Name'),
             '#description' => $this->t('Enter the name of the table to export data from.'),
             '#required' => TRUE,
             '#ajax' => [
                 'callback' => '::fetchTableHeaders',
                 'event' => 'change',
                 'wrapper' => 'table-headers-wrapper',
             ],
         ];
 
         $form['table_headers'] = [
             '#type' => 'container',
             '#attributes' => ['id' => 'table-headers-wrapper'],
         ];
 
         if (!empty($form_state->getValue('table_name'))) {
             $table_name = $form_state->getValue('table_name');
             $headers = $this->getTableColumns($table_name);
             if ($headers) {
                 foreach ($headers as $header) {
                     $form['table_headers']['header_' . $header] = [
                         '#type' => 'checkbox',
                         '#title' => $this->t('@header', ['@header' => $header]),
                         '#default_value' => TRUE,
                     ];
                 }
             } else {
                 $form['table_headers']['#markup'] = $this->t('Table not found or no records available.');
             }
         }
 
         $form['export_format'] = [
             '#type' => 'select',
             '#title' => $this->t('Select Export Format'),
             '#options' => [
                 'csv' => $this->t('CSV'),
                 'xlsx' => $this->t('XLSX'),
                 'pdf' => $this->t('PDF'),
                 'docx' => $this->t('DOCX'),
             ],
             '#required' => TRUE,
         ];
 
         $form['submit'] = [
             '#type' => 'submit',
             '#value' => $this->t('Export'),
         ];
 
         return $form;
     }
 
     public function fetchTableHeaders(array &$form, FormStateInterface $form_state)
     {
         $form_state->setRebuild(TRUE); // Ensures the form is rebuilt after AJAX
         return $form['table_headers'];
     }

    /**
     * Form submission handler.
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        // dd($form_state->getValue('table_headers'));
        $table_name = $form_state->getValue('table_name');
        $selected_headers = [];

        foreach ($form_state->getValues() as $key => $value) {
            if (strpos($key, 'header_') === 0 && $value) {
                $selected_headers[] = str_replace('header_', '', $key);
            }
        }

        if (empty($selected_headers)) {
            $this->messenger()->addError($this->t('Please select at least one column to export.'));
            return;
        }
// dd($selected_headers);
        $format = $form_state->getValue('export_format');
        $data = $this->fetchData($table_name, $selected_headers);

        // dd($data);
        // Generate the export based on the selected format
        switch ($format) {
            case 'csv':
                $this->dataExportService->exportTableCsv($data, $selected_headers, $table_name);
                break;
    
            case 'xlsx':
                $this->dataExportService->exportTableXlsx($data, $selected_headers, $table_name);
                break;
    
            case 'pdf':
                $this->dataExportService->exportTablePdf($data, $selected_headers, $table_name);
                break;
    
            case 'docx':
                $this->dataExportService->exportTableDocx($data, $selected_headers, $table_name);
                break;
        }
        // Logging selected headers
        \Drupal::logger('data_export')->notice('Selected headers: @headers', ['@headers' => implode(', ', $selected_headers)]);

        // Add logic to fetch data and handle export

        $this->messenger()->addStatus($this->t('Export initiated in @format format for table @table.', ['@format' => $format, '@table' => $table_name]));
    }
    /**
 * Fetch data from the specified table and columns.
 */
protected function fetchData($table_name, array $selected_headers) {
    $database = Database::getConnection();
    $query = $database->select($table_name, 't')
      ->fields('t', $selected_headers)
      ->execute();
  
    return $query->fetchAll(); // Adjust 'id' to the appropriate unique field if needed
  }
  

    /**
     * Helper function to fetch columns of a table.
     */
    protected function getTableColumns($table_name)
    {
        $columns = [];
        try {
            $database = Database::getConnection();
            $database_name = $database->getConnectionOptions()['database'];

            $query = $database->select('information_schema.columns', 'c')
                ->fields('c', ['COLUMN_NAME'])
                ->condition('c.TABLE_NAME', $table_name)
                ->condition('c.TABLE_SCHEMA', $database_name)
                ->execute();

            $columns = $query->fetchCol();
            // dd($columns);
        } catch (\Exception $e) {
            $this->messenger()->addError($this->t('Failed to fetch columns: @error', ['@error' => $e->getMessage()]));
        }
        return $columns;
    }
}
