<?php

namespace Drupal\data_export\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Database;
use Drupal\data_export\Service\DataExportService;
use Symfony\Component\DependencyInjection\ContainerInterface;

use TCPDF;


class ExportUsingCodeForm extends FormBase
{
  /**
   * The Data Export Service.
   *
   * @var \Drupal\data_export\Service\DataExportService
   */
  protected $dataExportService;

  /**
   * Constructs a new ExportUsingCodeForm object.
   *
   * @param \Drupal\data_export\Service\DataExportService $data_export_service
   *   The Data Export Service.
   */
  public function __construct(DataExportService $data_export_service) {
    $this->dataExportService = $data_export_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('data_export.data_export_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId()
  {
    return 'export_using_code_form';
  }

  /**
   * Builds the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state)
  {
    $form['#attached']['library'][] = 'data_export/datatables';  // Attach the library defined in step 1

    // SQL query input field
    $form['sql_query'] = [
      '#type' => 'textarea',
      '#title' => $this->t('SQL Query'),
      '#description' => $this->t('Enter a valid SQL query to fetch data for export.'),
      '#required' => TRUE,
    ];

    // Button to show records
    $form['show_records'] = [
      '#type' => 'submit',
      '#value' => $this->t('Show Records'),
      '#submit' => ['::showRecords'],
      '#ajax' => [
        'callback' => '::showQueryResults',
        'wrapper' => 'query-results-wrapper',
      ],
    ];

    // Placeholder for query results
    $form['query_results'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'query-results-wrapper'],
    ];

    // File format selection
    $form['export_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Select Export Format'),
      '#options' => [
        'csv' => $this->t('CSV'),
        'xlsx' => $this->t('XLSX'),
        'pdf' => $this->t('PDF'),
        'docx' => $this->t('DOCX'),
      ],
      '#required' => TRUE,
    ];

    // Submit button
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Export'),
    ];

    return $form;
  }

  /**
   * Handles the showing of query results.
   */
  public function showQueryResults(array &$form, FormStateInterface $form_state)
  {
    $sql_query = $form_state->getValue('sql_query');
    $results = $this->executeQuery($sql_query);

    if (!empty($results)) {
      // Convert stdClass objects to associative arrays
      $results_array = array_map(function ($item) {
        return (array) $item;
      }, $results);

      // Ensure the results are not empty before accessing keys
      $header = !empty($results_array) ? array_keys(reset($results_array)) : [];

      // Display results as a table (for simplicity, show the first 10 rows)
      $form['query_results'] = [
        '#type' => 'markup',
        '#markup' => '<table id="results-table" class="table table-striped">' .
          '<thead><tr><th>' . implode('</th><th>', $header) . '</th></tr></thead>' .
          '<tbody>' .
          implode('', array_map(function ($row) {
            return '<tr><td>' . implode('</td><td>', array_map('htmlspecialchars', $row)) . '</td></tr>';
          }, $results_array)) .
          '</tbody></table>',

      ];
    } else {
      $form['query_results'] = [
        '#markup' => $this->t('No records found.'),
      ];
    }

    return $form['query_results'];
  }


  /**
   * Form submission handler for showing records.
   */
  public function showRecords(array &$form, FormStateInterface $form_state)
  {
    // The functionality to show records is handled in the `showQueryResults` method.
  }

  /**
   * Form submission handler.
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $sql_query = $form_state->getValue('sql_query');
    $format = $form_state->getValue('export_format');

    // Execute the SQL query to get the results
    $results = $this->executeQuery($sql_query);

    if (!empty($results)) {
      // Convert stdClass objects to associative arrays
      $results_array = array_map(function ($item) {
        return (array) $item;
      }, $results);

      switch ($format) {
        case 'csv':
          $this->dataExportService->exportCsv($results_array);
          break;
        case 'xlsx':
          $this->dataExportService->exportXlsx($results_array);
          break;
        case 'pdf':
          $this->dataExportService->exportPdf($results_array);
          break;
        case 'docx':
          $this->dataExportService->exportDocx($results_array);
          break;
        default:
          $this->messenger()->addError($this->t('Unsupported format.'));
          return;
      }
      $this->messenger()->addStatus($this->t('Export initiated in @format format.', ['@format' => $format]));
    } else {
      $this->messenger()->addError($this->t('No data to export.'));
    }
  }


  /**
   * Executes the SQL query.
   */
  protected function executeQuery($sql_query)
  {
    try {
      $database = Database::getConnection();
      return $database->query($sql_query)->fetchAll();
    } catch (\Exception $e) {
      $this->messenger()->addError($this->t('Error executing query: @error', ['@error' => $e->getMessage()]));
      return [];
    }
  }
 }
