<?php

namespace Drupal\data_export\Service;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpWord\PhpWord;
use PhpOffice\PhpWord\IOFactory;
use TCPDF;

class DataExportService
{


    /**
     * Exports data as CSV.
     */
    public function exportCsv($data)
    {
        header('Content-Type: text/csv');
        header('Content-Disposition: attachment;filename="export.csv"');
        $output = fopen('php://output', 'w');
        fputcsv($output, array_keys(reset($data))); // Headers
        foreach ($data as $row) {
            fputcsv($output, $row);
        }
        fclose($output);
        exit();
    }

    /**
     * Exports data as XLSX.
     */
    public function exportXlsx($data)
    {
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();

        $sheet->fromArray(array_keys(reset($data)), NULL, 'A1'); // Headers
        $sheet->fromArray($data, NULL, 'A2'); // Data

        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="export.xlsx"');
        header('Cache-Control: max-age=0');

        $writer = new Xlsx($spreadsheet);
        $writer->save('php://output');
        exit();
    }

    /**
     * Exports data as PDF.
     */
    public function exportPdf($data)
    {
      
        try {
            // Create a new PDF instance
            $pdf = new TCPDF();
            $pdf->SetCreator(PDF_CREATOR);
            $pdf->SetAuthor('Your Name');
            $pdf->SetTitle('Exported Data');
            $pdf->SetSubject('PDF Export');
            $pdf->AddPage();

            // Start building the HTML for the table
            $html = '<h2>Exported Data</h2><table border="1" cellpadding="5"><thead><tr>';

            // Check if data is not empty
            if (!empty($data)) {
                // Extract headers dynamically from the first row
                $header = array_keys(reset($data));
                foreach ($header as $column) {
                    $html .= '<th>' . htmlspecialchars($column) . '</th>';
                }
                $html .= '</tr></thead><tbody>';

                // Add rows dynamically
                foreach ($data as $row) {
                    $html .= '<tr>';
                    foreach ($header as $column) {
                        $cell = isset($row[$column]) ? htmlspecialchars($row[$column]) : '';
                        $html .= '<td>' . $cell . '</td>';
                    }
                    $html .= '</tr>';
                }
                $html .= '</tbody></table>';
            } else {
                $html .= '<p>No data available to export.</p>';
            }

            // Write HTML to PDF
            $pdf->writeHTML($html, true, false, true, false, '');

            // Set headers for file download
            header('Content-Type: application/pdf');
            header('Content-Disposition: attachment;filename="export.pdf"');
            header('Cache-Control: max-age=0');
            header('Cache-Control: max-age=1');
            header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
            header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
            header('Pragma: public');

            // Output the PDF
            $pdf->Output('export.pdf', 'D'); // 'D' for download

            // Exit to ensure no further content is sent
            exit();
        } catch (\Exception $e) {
            $this->messenger()->addError($this->t('Error generating PDF: @error', ['@error' => $e->getMessage()]));
        }
    }
    /**
     * Exports data as DOCX.
     */
    public function exportDocx($data)
    {


        // Create a new PhpWord instance
        $phpWord = new PhpWord();
        $section = $phpWord->addSection();

        // Add a title to the document
        $section->addTitle('Exported Data', 1);

        // Create a table
        $table = $section->addTable([
            'borderSize' => 6, // Border thickness
            'cellMargin' => 80, // Cell margin
        ]);

        // Add header row to the table
        $header = array_keys(reset($data));
        $table->addRow();
        foreach ($header as $cell) {
            $table->addCell()->addText(htmlspecialchars($cell), ['bold' => true]);
        }

        // Add data rows to the table
        foreach ($data as $row) {
            $table->addRow();
            foreach ($header as $cell) {
                $table->addCell()->addText(htmlspecialchars($row[$cell] ?? ''));
            }
        }

        // Set headers for file download
        header('Content-Type: application/vnd.openxmlformats-officedocument.wordprocessingml.document');
        header('Content-Disposition: attachment;filename="export.docx"');
        header('Cache-Control: max-age=0');

        // Save the DOCX file to output
        $writer = IOFactory::createWriter($phpWord, 'Word2007');
        $writer->save('php://output');

        // Ensure no further content is sent
        exit();
    }


    /**
     * Exports table data as CSV.
     */
    public function exportTableCsv(array $data, array $headers, $table_name) {
        header('Content-Type: text/csv');
        header('Content-Disposition: attachment;filename="export.csv"');
        $output = fopen('php://output', 'w');
        fputcsv($output, $headers); // Write headers
        foreach ($data as $row) {
            fputcsv($output, (array) $row); // Write data rows
        }
        fclose($output);
        exit();
    }

    /**
     * Exports table data as XLSX.
     */
    public function exportTableXlsx(array $data, array $headers, $table_name) {
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();

        $sheet->fromArray($headers, NULL, 'A1'); // Headers
        $sheet->fromArray(array_map('get_object_vars', $data), NULL, 'A2'); // Data

        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="export.xlsx"');
        header('Cache-Control: max-age=0');

        $writer = new Xlsx($spreadsheet);
        $writer->save('php://output');
        exit();
    }

    /**
     * Exports table data as PDF.
     */
    public function exportTablePdf(array $data, array $headers, $table_name) {
        try {
            $pdf = new TCPDF();
            $pdf->SetCreator(PDF_CREATOR);
            $pdf->SetAuthor('Your Name');
            $pdf->SetTitle('Exported Data from ' . $table_name);
            $pdf->SetSubject('PDF Export');
            $pdf->SetHeaderData('', 0, $table_name . ' Data Export', '');
            $pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
            $pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));
            $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
            $pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
            $pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);
            $pdf->AddPage();

            $html = '<h1>' . $table_name . ' Export</h1>';
            $html .= '<table border="1" cellspacing="0" cellpadding="5">';
            $html .= '<tr>';
            foreach ($headers as $header) {
                $html .= '<th>' . $header . '</th>';
            }
            $html .= '</tr>';

            foreach ($data as $row) {
                $html .= '<tr>';
                foreach ($row as $cell) {
                    $html .= '<td>' . htmlspecialchars($cell) . '</td>';
                }
                $html .= '</tr>';
            }

            $html .= '</table>';
            $pdf->writeHTML($html, true, false, true, false, '');

            $pdf->Output('export.pdf', 'D');
            exit();
        } catch (\Exception $e) {
            drupal_set_message(t('Error generating PDF: @message', ['@message' => $e->getMessage()]), 'error');
        }
    }

    /**
     * Exports table data as DOCX.
     */
    public function exportTableDocx(array $data, array $headers, $table_name) {
        $phpWord = new PhpWord();
        $section = $phpWord->addSection();

        // Add a title
        $section->addText(
            $table_name . ' Export',
            ['bold' => true, 'size' => 16],
            ['alignment' => 'center']
        );

        // Add some space after the title
        $section->addTextBreak(1);

        // Add a table with predefined width
        $table = $section->addTable([
            'borderSize' => 6, // Table border size
            'borderColor' => '999999', // Border color (gray)
            'cellMargin' => 80, // Cell margin
            'alignment' => \PhpOffice\PhpWord\SimpleType\JcTable::CENTER, // Center the table
        ]);

        // Add header row with styling
        $table->addRow();
        foreach ($headers as $header) {
            $table->addCell(2000, ['bgColor' => 'DDDDDD'])->addText($header, ['bold' => true, 'color' => '000000']);
        }

        // Add data rows
        foreach ($data as $row) {
            $table->addRow();
            foreach ($row as $cell) {
                $table->addCell(2000)->addText($cell);
            }
        }

        // Stream the DOCX file to the browser
        header('Content-Type: application/vnd.openxmlformats-officedocument.wordprocessingml.document');
        header('Content-Disposition: attachment;filename="export.docx"');

        $objWriter = IOFactory::createWriter($phpWord, 'Word2007');
        $objWriter->save('php://output');
        exit();

    }
}
